import matplotlib.pyplot as pyplot

import const


class Section(object):  # TODO: rename to structure?

    def __init__(self, fragment_list, onset=0.0):

        #: List. A list of Fragment objects.
        self.fragment_list = fragment_list

        self._onset = None
        #: Float. The time of onset. The unit of measurement is abstract, so
        #: the user may interpret *onset* however they wish, e.g. absolute
        #: time, musical time, etc.
        self.onset = onset

    @property
    def onset(self):
        return self._onset

    @onset.setter
    def onset(self, value):
        self._onset = value
        self.sort()
        self.queue()

    # TODO: magic methods

    def __repr__(self):
        s = ""
        for fragment in self.fragment_list:
            s += fragment.__repr__()
            s += "\n"
        return s

    # plot
    # ------------------------------------------------------------------- #

    def plot(self):
        """
        Plot *fragment_list* with matplotlib.

        """
        for fragment in self.fragment_list:
            # extract x-axis data
            x = fragment.onset_list()
            # extract y-axis data
            y = fragment.pitch_list()
            # plot
            pyplot.plot(x, y, 'ro', ls='steps-post-')
        # axis
        pyplot.axis([self.onset - 1,
                     self.fragment_list[0].onset + self.duration() + 1,
                     const.MIN_MIDI_NUMBER,
                     const.MAX_MIDI_NUMBER])
        pyplot.xlabel(const.X_LABEL)
        pyplot.ylabel(const.Y_LABEL)
        # show plot(s)
        pyplot.show()

    # info
    # ------------------------------------------------------------------- #

    def duration(self):
        """
        Return duration of Section object.

        :returns: float -- duration

        """
        duration = 0.0
        for fragment in self.fragment_list:
            duration += fragment.duration()
        return duration

    # sort
    # ------------------------------------------------------------------- #

    def queue(self):
        """
        Queue fragment objects in *fragment_list* relative to *onset*.

        """
        o = self.onset
        for fragment in self.fragment_list:
            fragment.onset = o
            o += fragment.duration()

    def sort(self):
        """
        Sort *fragment_list* by *onset*.

        """
        self.fragment_list.sort(key=lambda x: x.onset)
